﻿#!/usr/bin/python
# -*- coding: utf-8 -*-
"""『SS7 Op.Ptyhon実行』活用サンプル 「応力経路の描画」

このモジュールは『SS7 Op.Ptyhon実行』を活用していただくためのサンプルプログラムです。
指定したフォルダ内にある『SS7』の入力データCSVから、解析ステップごとに物件データを作成し解析し、
解析結果から、指定した柱の部材応力を取得し、MN図に描画するプログラムとなっています。
このサンプルプログラムにより、『SS7 Op.Ptyhon実行』の使用方法として、
・『SS7』の物件データの基本的な扱い方
・『SS7』にコマンドを送信して動作させる方法
・『SS7』CSVデータの変更方法
を学ぶことができます。
また、複数の『SS7』物件データに対して連続的に何かの処理を行わせたいバッチ処理のための雛形としても活用いただけます。

"""

# 『SS7 Op.Ptyhon実行』パッケージ
from Python import Ss7Python as ss7cmd

# CSVファイル操作、耐力取得で用いるモジュール
import csv

# MN曲線、応力経路を描画するモジュール
import matplotlib.pyplot as plt

# --------------------------------------------------------------
# 設定

# 入力CSVのフルパス
input_csv: str = r"C:\UsrData\SS7CSV\Sample1.csv"
# 出力用フォルダのフルパス
ouput_folder: str = r"C:\UsrData\SS7CSV\OutData"

# 対象ケース
target_case: str = "case=DSX+"  # ケース

# ステップ数
max_stepno: int = 50  # 最大ステップ数
step_pitch: int = 2  # ステップ間隔

# 応力経路を確認する柱
floor: str = "2"  # 階
jiku1: str = "101"  # 軸1
jiku2: str = "3"  # 軸2
is_x: bool = True  # X方向の場合はTrue、Y方向の場合はFalse
is_top: bool = True  # 柱頭の場合はTrue、柱脚の場合はFalse


# --------------------------------------------------------------
# CSVファイルを作成する
def make_csvfile(step_no: int, input_filename: str, csv_filename: str) -> None:
    """make_csvfile 関数

    解析ステップ数を変更した入力データCSVを新規に作成します。

    Args:
        step_no (int):解析ステップ数
        input_filename(str):入力データCSVファイル名
        csv_filename(str):解析ステップ数を変更した入力データCSVファイル名

    """
    teigi_name: str = "Ds算定時の定義"
    if "Q" in target_case:
        teigi_name = "保有水平耐力時の定義"

    case_no: int = 0  # 0=正加力X加力時
    if "X-" in target_case:
        case_no = 2  # 2=負加力X加力時
    elif "Y+" in target_case:
        case_no = 1  # 1=正加力Y加力時
    elif "Y-" in target_case:
        case_no = 3  # 3=負加力Y加力時

    # 特定の行を識別するキーワード
    keyword: str = "最大ステップ数"

    # CSVファイルを読み込む
    with open(input_filename, "r") as input_file:
        reader = csv.reader(input_file)
        data = list(reader)

    # データを変更する
    taige_name_row: bool = False
    processing_row: bool = False
    for i, row in enumerate(data):
        if teigi_name in row:
            taige_name_row = True
        if taige_name_row:
            if keyword in row:
                processing_row = True
            if processing_row:
                if i + case_no < len(data):
                    next_row = data[i + case_no]
                    next_row[-1] = str(step_no)
                    break

    # 変更したデータを新しいCSVファイルに書き込む
    with open(csv_filename, "w", newline="") as output_file:
        writer = csv.writer(output_file)
        writer.writerows(data)


# --------------------------------------------------------------
# 柱の応力結果の取得
def get_column_stress(output_csv_path: str) -> tuple[float, float]:
    """get_column_stress 関数

    設定で指定した柱の曲げモーメントと軸力を取得します。

    Args:
        output_csv_path(str):結果データCSVファイルのパス

    Returns:
        tuple[float, float]:指定した柱の曲げモーメント,指定した柱の軸力

    """
    # 特定の行を識別するキーワード
    keyword: str = "<data>"

    n_idx: int = 12  # 軸力N
    m_idx: int = 4  # 曲げモーメントM(X方向・柱頭)
    x_to_y: int = 4  # X方向→Y方向への変換
    t_to_b: int = 1  # 柱頭→柱脚への変換

    if is_x is False:
        m_idx += x_to_y
    if is_top is False:
        m_idx += t_to_b

    # CSVファイルを読み込む
    with open(output_csv_path, "r") as output_file:
        reader = csv.reader(output_file)
        data = list(reader)

    # データを検索する
    case_row: bool = False
    processing_row: bool = False
    for i, row in enumerate(data):
        if target_case in row:
            case_row = True
        if case_row:
            if keyword in row:
                processing_row = True
            if processing_row:
                if floor == row[0] and jiku1 == row[1] and jiku2 == row[2]:
                    return (float(row[m_idx]) * -1.0, float(row[n_idx]))

    # 対象データが見つからないとき
    return 0.0, 0.0


# --------------------------------------------------------------
# 柱の耐力結果の取得
def get_column_strength(output_csv_path: str) -> list:
    """get_column_strength 関数

    設定で指定した柱のMN耐力曲線を取得します。

    Args:
        output_csv_path(str):結果データCSVファイルのパス

    Returns:
        list:[曲げモーメント,軸力]を1組としたMN耐力曲線のリスト

    """
    # 特定の列を識別するキーワード
    keyword: str = "<data>"  # このキーワード以降にMNデータが存在する
    keyword1: str = "(対称) "
    keyword2: str = "左側"

    # CSVファイルを読み込む
    with open(output_csv_path, "r") as output_file:
        reader = csv.reader(output_file)
        data = list(reader)

    # 読み込み位置指定用追加インデックス
    add_index: int = 0
    if is_x is False:
        add_index += 3
    if is_top is False:
        add_index += 2

    # 曲げデータを取得する
    mn_strength: list = []
    processing_row: bool = False
    end_flag: bool = False
    taisho_flag: bool = False
    for i, row in enumerate(data):
        # MNデータが存在する行となったか？
        if keyword in row:
            processing_row = True
        if processing_row:
            # 目的の柱部材か？
            if floor == row[0] and jiku1 == row[1] and jiku2 == row[2]:
                traget_row = data[i + add_index]
                for idx in range(7, 31):
                    #  "(対称) "の文字列のときMNの取得を終了する
                    if keyword1 in traget_row[idx]:
                        end_flag = True
                        taisho_flag = True
                        break
                    # "左側"の文字列のときMNの取得を終了する
                    if keyword2 in traget_row[idx]:
                        end_flag = True
                        break
                    mn_strength.append(
                        (float(traget_row[idx]), float(data[i + add_index + 1][idx]))
                    )
                if end_flag:
                    break

    # "(対称) "の場合、負側のMNデータを作成し登録する
    if taisho_flag:
        add_strength = []
        for x in mn_strength[::-1]:
            add_strength.append((-x[0], x[1]))
        mn_strength.extend([x for x in add_strength])

    return mn_strength


# --------------------------------------------------------------
# メイン処理
# --------------------------------------------------------------
ss7cmd.Init()
ss7cmd.Start(None, 1)  # 1=ログファイルをクリアする

mn_data_list: list = []  # NとMを格納するリスト
mn_strength: list = []  # MNの耐力を格納するリスト

first_flag: bool = True

for Step in range(1, max_stepno + 1, step_pitch):
    # CSVファイルを作成する
    csv_path = ouput_folder + r"\output_" + str(Step) + ".csv"
    make_csvfile(Step, input_csv, csv_path)

    # CSV入力データからSS7のデータを作成する
    dataname = ouput_folder + r"\Sample1_" + str(Step) + "Step.ikn"
    ss7cmd.CreateDataCsv(csv_path, dataname, 2)  # 2=中断する

    # エラー情報を取得する
    err = ss7cmd.GetLastError()

    # CSV新規作成できたことの判定
    if err.IsOK() is True:
        print("CSV新規作成が成功しました")

        # データを開く
        data = ss7cmd.Open(dataname, 1, 3)  # 1=変換して開く 3=物件データは開かない

        # データが正しく開けたことの判定
        if data is not None:

            # 計算実行
            data.Calculate("結果1", "応力解析（二次）")

            # エラー情報を取得する
            err = ss7cmd.GetLastError()

            # 計算が正しく実行できたことの判定
            if err.IsOK() is True:
                print("計算が成功しました")

                # 計算結果データを取得する
                res = data.GetResultData("結果1")

                # 結果CSVをエクスポートする
                res.ExportResultCsv(
                    "12.21.2.柱", dataname + "\\Result1.csv", 2, 1, 1
                )  # 2=中断する 1=省略する 1=全部材出力

                err = ss7cmd.GetLastError()
                # 結果CSVのエクスポートが正しく実行できたことの判定
                if err.IsOK() is True:
                    print("結果CSVのエクスポートが成功しました")
                    # 柱の応力結果を取得する
                    mn_data_list.append(get_column_stress(dataname + "\\Result1.csv"))
                else:
                    print("結果CSVのエクスポートが失敗しました")

                if first_flag:
                    # 結果CSVをエクスポートする
                    res.ExportResultCsv(
                        "11.3.終局耐力（柱）", dataname + "\\Result2.csv", 2, 1, 1
                    )  # 2=中断する 1=省略する 1=全部材出力

                    err = ss7cmd.GetLastError()
                    # 結果CSVのエクスポートが正しく実行できたことの判定
                    if err.IsOK() is True:
                        print("結果CSV（耐力）のエクスポートが成功しました")
                        # 柱の耐力結果を取得する
                        mn_strength = get_column_strength(dataname + "\\Result2.csv")
                        first_flag = False
                    else:
                        print("結果CSV（耐力）のエクスポートが失敗しました")

                # データを閉じる
                data.Close(1)  # 1=保存する

            else:
                print("計算が失敗しました")

                # データを閉じる
                data.Close(2)  # 2=保存しない
    else:
        print("CSV新規作成が失敗しました")

# すべて終わったので終了する
ss7cmd.End(1)  # 1=保存して終了

# ----------------------------------------------------------------
# NとMの応力経路を描画

# NMdata_listからNとMの値をそれぞれ別のリストに分ける
m_values: list = [float(mn_d[0]) for mn_d in mn_data_list]
n_values: list = [float(mn_d[1]) for mn_d in mn_data_list]

# x=0のラインを描画
plt.axvline(0, color="r")  # 'r'は赤色を意味します
# y=0のラインを描画
plt.axhline(0, color="r")  # 'r'は赤色を意味します

# 散布図を作成
plt.scatter(m_values, n_values)

# 点を順番に線で結ぶ
plt.plot(m_values, n_values)

# グラフのタイトルと軸ラベルを設定
plt.title("MN path view")
plt.xlabel("M")
plt.ylabel("N")

# 耐力線を描画
m_strength = [mn_s[0] for mn_s in mn_strength]
n_strength = [mn_s[1] for mn_s in mn_strength]
plt.plot(m_strength, n_strength)

# グラフをJPEGで保存

plt.savefig((ouput_folder + r"\MNpath.jpg"), format="jpeg")

# グラフを表示
plt.show()
